#! /bin/bash
# Generated by tor.SlackBuild for tor-0.1.2.19-i486-1McD.tgz on 31-Jan-2008

function upd_conf {
 	# If there's no config file by that name, mv it over:
 	if ! test -e $1; then
 		mv $1.new $1
 	elif diff -q $1 $1.new; then	# toss the redundant copy
 		rm $1.new
 	elif ! test -s $1.new; then
 		rm $1.new   # do not keep empty initial files (e.g. log)
 	fi
 	# Otherwise, we leave the .new copy for the admin to consider...
}

function free_user_id {
 	# Find a free user-ID >= 100 (should be < 1000 so it's not a normal user)
 	local FREE_USER_ID=100
 	while grep -q "^.*:.*:$FREE_USER_ID:.*:.*:.*:" etc/passwd; do
 		let FREE_USER_ID++
 	done
 	echo $FREE_USER_ID
}

# Edit config file with currently used user/group
# (it's not worth leaving a .new file over)
if test -f etc/tor/torrc && ! grep -qE \
 	'^User[[:blank:]]+tor_d[[:blank:]]*$' etc/tor/torrc
then
 	printf 'Changing user in /etc/tor/torrc to \033[1mtor_d\033[0m.\n' 1>&2
 	TMPFILE=$(mktemp /tmp/tor.XXXXXX)
 	sed '	/^User[[:blank:]]/	s/[[:blank:]].*/ tor_d/
 		/^Group[[:blank:]]/	s/[[:blank:]].*/ daemon/
 		' etc/tor/torrc > $TMPFILE
 	cat $TMPFILE > etc/tor/torrc
 	rm $TMPFILE
fi

# Edit logrotate.d file with currently used user/group
# (it's not worth leaving a .new file over)
if test -f etc/logrotate.d/tor && ! grep -qE \
 	'^[[:blank:]]*create[[:blank:]]+[0-7][0-7][0-7][0-7]?[[:blank:]]+tor_d[[:blank:]]+daemon[[:blank:]]*$' \
 	etc/logrotate.d/tor
then
 	printf 'Changing user in /etc/logrotate.d/tor to \033[1mtor_d\033[0m.\n' 1>&2
 	TMPFILE=$(mktemp /tmp/tor.XXXXXX)
 	sed '	/^[[:blank:]]*create/	s/\([0-7][[:blank:]]\).*/\1tor_d daemon/
 		' etc/logrotate.d/tor > $TMPFILE
 	cat $TMPFILE > etc/logrotate.d/tor
 	rm $TMPFILE
fi

# Set up group if needed
grep -q '^daemon:' etc/group || chroot . /usr/sbin/groupadd daemon

# Set up user: add it if it doesn't exist, update it if it already does.
# In any case, make sure it has or gets a user ID between 100 and 999.
# If username changed, determine old username so we can change it (look
# for entry with homedir /var/{log,run,lib}/tor).
printf 'Setting up unprivileged user \033[1mtor_d\033[0m to run the daemon.\n'
if OLD_ENTRY=$(grep -m 1 '^tor_d:' etc/passwd)		\
 	|| OLD_ENTRY=$(grep -m 1				\
 		':/var/[lr][oui][gnb]/tor:[a-z/]*$' etc/passwd)
then
 	# Modify existing user
 	OLD_USER=$(echo $OLD_ENTRY | cut -f 1 -d ':')
 	USER_ID=$(echo $OLD_ENTRY | cut -f 3 -d ':')
 	test $USER_ID -ge 1000 && USER_ID=$(free_user_id)
 	chroot . /usr/sbin/usermod -l tor_d -g daemon		\
 		-d /var/run/tor -u $USER_ID -s /bin/false		\
 		-c "Tor Internet Anonymizer" $OLD_USER
else
 	# Add new user
 	chroot . /usr/sbin/useradd -g daemon			\
 		-d /var/run/tor -u $(free_user_id) -s /bin/false	\
 		-c "Tor Internet Anonymizer" tor_d
fi

# Set up permissions
chroot . /bin/chown -R tor_d:daemon /var/log/tor	\
/var/lib/tor /var/run/tor

# Make us start on boot
if ! grep -q "rc.tor start" etc/rc.d/rc.local
then
 	echo 'Adding Tor to /etc/rc.d/rc.local so it starts at bootup time.' 1>&2
 	{	echo ''
 		echo '# Tor, the Internet anonymizer'
 		echo 'test -x /etc/rc.d/rc.tor && /etc/rc.d/rc.tor start'
 	} >> etc/rc.d/rc.local
else
 	echo 'Tor is already in /etc/rc.d/rc.local .'
fi

# Make us stop on shutdown
if test ! -e etc/rc.d/rc.local_shutdown; then
 	touch etc/rc.d/rc.local_shutdown
 	chmod +x etc/rc.d/rc.local_shutdown
fi
if ! grep -q "rc.tor stop" etc/rc.d/rc.local_shutdown 2>/dev/null; then
 	echo 'Adding Tor to /etc/rc.d/rc.local_shutdown so it stops at shutdown time.' 1>&2
 	{	echo ''
 		echo '# Tor, the Internet anonymizer'
 		echo 'test -x /etc/rc.d/rc.tor && /etc/rc.d/rc.tor stop'
 	} >> etc/rc.d/rc.local_shutdown
else
 	echo 'Tor is already in /etc/rc.d/rc.local_shutdown .'
fi

upd_conf etc/logrotate.d/tor
upd_conf etc/rc.d/rc.tor
upd_conf etc/tor/tor-tsocks.conf
upd_conf etc/tor/torrc
upd_conf var/log/tor/notices.log > /dev/null 2>&1
